/******************************************************************************
* AndroidServer.h - header file for AndroidServerClass.	                      *
* This class implments the server that talks with android device.             *
*******************************************************************************
* (C) Gershon Elber, Technion, Israel Institute of Technology                 *
*******************************************************************************
* Written by Moran Dringer and Shiran Maor, January 2013.		      *
******************************************************************************/

#ifndef ANDROID_SERVER_H
#define ANDROID_SERVER_H

#include "AndroidServerDefs.h"

#ifdef ExportGuIritServer
	#define DllExpImp __declspec( dllexport )
#else
	#define DllExpImp __declspec( dllimport )
#endif

enum {
    ANDRD_RESULT_ERROR = -1,
    ANDRD_RESULT_SUCCESS,
    ANDRD_RESULT_NO_MESSAGE
} typedef AndrdResultType; 

class DllExpImp AndroidServerClass {
    protected:
        AndroidCommandHandler TheCommandHandler;
        AndroidPrintFunction ThePrintFunction;
	bool MouseEmulationActive;
        unsigned int MaxEventsPerSecond;
    public:
        AndroidServerClass(void);
        virtual ~AndroidServerClass(void) = 0;
        static AndroidServerClass *GetBluetoothServer();
	void PushCommandHandler(AndroidCommandHandler TheHandlerFunction);
	void PopCommandHandler();
	AndroidPrintFunction SetPrintFunction(AndroidPrintFunction ThePrintFunc);
	bool ServerSelectObject();
	bool ServerMouseEmulation(bool MouseEmulation);
	bool ServerMouseEmulation() {
	    return MouseEmulationActive;
	}
	int SetMaxEventsPerSecond(unsigned int TheMaxEventsPerSecond);

        virtual AndrdResultType InitCommunication() {return ANDRD_RESULT_ERROR;}
        virtual AndrdResultType OnConnecting()      {return ANDRD_RESULT_ERROR;}
        virtual AndrdResultType OnConnected()       {return ANDRD_RESULT_ERROR;}
        virtual AndrdResultType OnDisconnecting()   {return ANDRD_RESULT_ERROR;}
}; 

bool ExecCommandHandlerStack(AndrdCommandDataStruct CommandData);

#endif /* ANDROID_SERVER_H */
