/******************************************************************************
* AndroidServerDefs.h - Holds defintions of the android commands.             *
* This class is used by AndroidServer(DLL) and the two extension.             *
*******************************************************************************
* (C) Gershon Elber, Technion, Israel Institute of Technology                 *
*******************************************************************************
* Written by Moran Dringer and Shiran Maor, January 2013.		      *
******************************************************************************/

#ifndef ANDROID_SERVER_DEFS_H
#define ANDROID_SERVER_DEFS_H

#define MAX_LENGTH_BUTTON_LABEL 25
#define MAX_LENGTH_SENSOR_NAME 25

#define MAX_FINGERS_NUMBER 10			      /* Should be at least 3 */

enum {
    ANDRD_CMD_TYPE_UNDEFINED = -1,
    ANDRD_CMD_TYPE_GPS = 0,
    ANDRD_CMD_TYPE_ACCELEROMETER,
    ANDRD_CMD_TYPE_MAGNETIC_FIELD,
    ANDRD_CMD_TYPE_ORIENTATION,
    ANDRD_CMD_TYPE_GYRO_SCOPE,
    ANDRD_CMD_TYPE_LIGHT,
    ANDRD_CMD_TYPE_PRESSURE,
    ANDRD_CMD_TYPE_TOUCH,
    ANDRD_CMD_TYPE_PROXIMITY,
    ANDRD_CMD_TYPE_GRAVITY,
    ANDRD_CMD_TYPE_LINEAR_ACCELEROMETER,
    ANDRD_CMD_TYPE_ROTATION_VECTOR,
    ANDRD_CMD_TYPE_BUTTON,
    ANDRD_CMD_TYPE_PHONE_BUTTON,
    ANDRD_CMD_TYPE_SHUTDOWN = 100
} typedef AndrdCmdType; 

enum {
    ANDRD_TOUCH_DOWN = 0,
    ANDRD_TOUCH_UP,
    ANDRD_TOUCH_MOVE,
    ANDRD_TOUCH_CLICK,
    ANDRD_TOUCH_DOUBLE_CLICK,
    ANDRD_TOUCH_TRIPLE_CLICK
} typedef AndrdTouchType;  

/* A struct to Get & Set AndroidServer data. */
struct AndrdCommandDataStruct {
    AndrdCmdType CmdType;
    char SensorName[MAX_LENGTH_SENSOR_NAME];
    long Timestamp;
    union {
	struct {
	    double Attitude;
	    double Latitude;
	    double Longitude;
	    double Speed;
	    double Accuracy;
	} GPS;	
	struct {
	    double AfX, AfY, AfZ;	       /* Acceleration force (m/s2). */
	} Accelerometer;
	struct {
	    double AgfX, AgfY, AgfZ;/*Ambient geomagnetic field (micro-Tesla).*/
	} MagneticField;	
	struct {
	    double TetaX, TetaY, TetaZ;				 /* Radians. */
	} Orientation;	
	struct {
	    double RorX, RorY, RorZ;	    /* Rate of rotations (Radian/s). */
	} GyroScope;	
	struct {
	    double Level;			       /* Illumination (lx). */
	} Light;	
	struct {
	    double AirPressure;		      /* Ambient air pressure (hPa). */
	} Pressure;
	struct {
	    AndrdTouchType Type;
	    int NumberOfFingers;
	    double X[MAX_FINGERS_NUMBER];
	    double Y[MAX_FINGERS_NUMBER];
	} Touch;	
	struct {
	    double Distance;
	} Proximity;	
	struct {
	    double FogX, FogY, FogZ;             /* Force of gravity (m/s2). */
	} Gravity;		
	struct {
	    double AfX, AfY, AfZ;	       /* Acceleration force (m/s2). */
	} LinearAcceleration;	
	struct {
	    double TetaX, TetaY, TetaZ;				 /* Radians. */
	} RotationVector;	
	struct {
	    int ID, State;
	    char Label[MAX_LENGTH_BUTTON_LABEL];
	} Button;	
	struct {
	    int ID;
	    char Label[MAX_LENGTH_BUTTON_LABEL];
	} PhoneButton;
    } AndrdCmdUnion;	
}; 

typedef bool (*AndroidCommandHandler)(AndrdCommandDataStruct CommandData);
typedef void (*AndroidPrintFunction)(const char *);

#endif /* ANDROID_SERVER_DEFS_H */
