#ifndef GC_COVERAGE_PARAMETERS_H
#define GC_COVERAGE_PARAMETERS_H

/****************************************************************************/

#include <setjmp.h>
#include "GC.h"

/****************************************************************************/
/* Global variables *********************************************************/
/****************************************************************************/

static jmp_buf CoverageInputDataCreatorLongJumpPoint;
static jmp_buf CoverageSolverLongJumpPoint;

/*****************************************************************************
*** Platform independent definitions and type definitions ********************
*****************************************************************************/

/* Definitions **************************************************************/

#define GC_WORK_DIMENSION                              1
#define GC_HALF_WARP_SIZE                              1
#define GC_NR_OF_RESULT_BUFFERS                        2

#define GC_MAXIMUM_INPUT_ELEMENT_CARDINALITY           16

/* strlen(x) + 1 (+ 1 for null-termination),
   where x in { "uchar", "ushort", "uint", "ulong" }
   (integer types supported by OpenCL)                                  */
#define GC_DEVICE_WORK_ITEM_TYPE_STRING_MAX_LENGTH     7

/* nrOfDigits(x) + 1 (+ 1 for null-termination),
   where x = GC_MAXIMUM_INPUT_ELEMENT_CARDINALITY (maximum cardinality of
   vectors supported by the current OpenCL version)                     */
#define GC_DEVICE_WORK_ITEM_CARDINALITY_STRING_MAX_LENGTH 3

#define GC_NR_OF_BITS_IN_HOST_WORK_ITEM   (GC_NR_OF_BITS_IN_BYTE * sizeof(GC_HostWorkItemType))

#define GC_MAX_NR_OF_DIGITS_IN_LARGEST_SIGNED_INTEGER_TYPE 19

#define GC_MAX_CARDINALITY_GREEDY                      (1u)

#define GC_MAX_INFO_STRING_LENGTH                      (1000u)
#define GC_MAX_SOURCE_CODE_LENGTH                      (50000u)

#define GC_NR_OF_BITS_IN_RESULT_ELEMENT_TYPE           (GC_NR_OF_BITS_IN_BYTE * sizeof(GC_ResultElementType))
#define GC_MAX_VALUE_OF_RESULT_ELEMENT_TYPE            GC_MAX_VALUE_OF_CL_UINT

#define GC_NR_OF_ITERATION_TYPE_STRING                 "uint"

#define GC_KERNEL_PROGRAM_SOURCE              "Kernels/KernelProgramSource.cl"

#define GC_COMPUTE_SUMS_KERNEL_NAME              "ComputeSums"
#define GC_COMPUTE_BEST_SUM_KERNEL_NAME          "ComputeBestSum"
#define GC_RESET_AUXILIARY_BUFFER0_KERNEL_NAME   "ResetResultBuffer"
#define GC_COMPUTE_TARGET_VECTOR_SUM_KERNEL_NAME "ComputeTargetVectorSum"
#define GC_COMPUTE_MAX_COVERAGE_KERNEL_NAME      "ComputeMaxCoverage"
#define GC_SUBTRACT_INPUT_KERNEL_NAME            "SubtractInputVector"

#define GC_INFINTY                            (2.3197171528332553e+25)

#define GC_IS_EXHAUSTIVE(x)                   (GC_EXHAUSTIVE == x -> AlgorithmType)
#define GC_IS_GREEDY(x)                       (GC_GREEDY == x -> AlgorithmType)

#define GC_NR_OF_ITERATIONS(x)      (x -> Optimization.NrOfIterations)
#define GC_USE_COMPRESSION(x)       (x -> Optimization.UseCompressedInputVectors)
#define GC_SORTING_SPEED_UP(x)      (x -> Optimization.SortingSpeedUp)

/* Type definitions *********************************************************/

typedef cl_uint                       GC_ResultElementType;                   /* has to be in {cl_uint, cl_ulong} */
typedef cl_int                        GC_OpenCLErrorType;

/* Enumerations **************************************************************
*/                                                                          /*
*   GC_NR_OF_KERNELS has to remain last entry!                               *
*/  typedef enum {                                                          /*
*/                                                                          /*
*/      GC_COMPUTE_SUMS_KERNEL,                                             /*
*/      GC_RESET_AUXILIARY_BUFFER0_KERNEL,                                  /*
*/      GC_NR_OF_KERNELS                                                    /*
*/                                                                          /*
*/  } GC_KernelsEnum;                                                       /*
*/                                                                          /*
*/  typedef enum {                                                          /*
*/                                                                          /*
*/      GC_SUBTRACT_INPUT_VECTOR_KERNEL = GC_NR_OF_KERNELS,                 /*
*/      GC_NR_OF_KERNELS_GREEDY,                                            /*
*/      GC_MAXIMAL_NR_OF_KERNELS        = GC_NR_OF_KERNELS_GREEDY           /*
*/                                                                          /*
*/  } GC_GreedyKernelsEnum;                                                 /*
*/                                                                          /*
*/  typedef enum {                                                          /*
*/                                                                          /*
*/      GC_NON_DESCENDING,                                                  /*
*/      GC_NON_ASCENDING                                                    /*
*/                                                                          /*
*/  } GC_SortingOrderEnum;                                                  /*
*/                                                                          /*
*****************************************************************************/

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

GC_BooleanType CheckCoverageParameters
    (      GC_CoverageParametersStruct*  Parameters,
     const void                          (*Printf)(GC_CharacterType*,
                                                   GC_MessageKind));

void DestroyCoverageParameters
    (      GC_CoverageParametersStruct** Parameters);

/****************************************************************************/

#endif /* GC_COVERAGE_PARAMETERS_H */