#ifndef GC_COVERAGE_SOLVER_INPUT_DATA_H
#define GC_COVERAGE_SOLVER_INPUT_DATA_H

/****************************************************************************/

#include "CoverageParameters.h"
#include "Definitions.h"

/****************************************************************************/
/* Structures ***************************************************************/
/****************************************************************************/

typedef struct {

    GC_HostWorkItemType**                  InputVectors;
    GC_LargestUnsignedIntegerType*         WeightsVector;
    GC_HostWorkItemType*                   TargetVector;

    GC_LargestUnsignedIntegerType          NrOfInputVectorWorkItemsOnHost;
    GC_LargestUnsignedIntegerType          NrOfTargetVectorWorkItemsOnHost;
    GC_LargestUnsignedIntegerType          NrOfWorkItemsOnDevice;
    GC_LargestUnsignedIntegerType          MaxWeight;
    cl_double                              MaximalPossibleCoverage;
    cl_double                              MaximalPossibleCoverageBeforeRoundingWeights;

    GC_LargestUnsignedIntegerType          NrOfVectorEntries;
    GC_LargestUnsignedIntegerType          NrOfCoverableTargetLocations;
    GC_LargestUnsignedIntegerType          NrOfUnsatisfiableElements;
    GC_LargestUnsignedIntegerType          AllowedNrOfUnsatisfiedVectorEntries;

    GC_NrOfWeightsType                     NrOfDifferentWeights;
    GC_NrOfWeightsType*                    IndexOfLastWeightInInterval;
    GC_NrOfWeightsType*                    SortedDifferentWeights;

    GC_NrOfWeightsType                     NrOfDifferentWeightsOfWorkItemsAccessingLocalMemory;

    GC_HostWorkItemType                    MaximalTargetVectorEntry;

    GC_BandWidthType                       NrOfBitsInDeviceVectorWorkItem;

    GC_BooleanType                         EndianMode;

    GC_CharacterType                       InputVectorDeviceWorkItemTypeString[GC_DEVICE_WORK_ITEM_TYPE_STRING_MAX_LENGTH + GC_DEVICE_WORK_ITEM_CARDINALITY_STRING_MAX_LENGTH];
    GC_CharacterType                       DeviceWorkItemCardinalityString[GC_DEVICE_WORK_ITEM_CARDINALITY_STRING_MAX_LENGTH];

    GC_CharacterType                       TargetVectorDeviceWorkItemTypeString[GC_DEVICE_WORK_ITEM_TYPE_STRING_MAX_LENGTH + GC_DEVICE_WORK_ITEM_CARDINALITY_STRING_MAX_LENGTH];

    GC_CharacterType*                      ResultElementTypeString;
    GC_CharacterType*                      ResultElementTypeMaxString;

    GC_BandWidthType                       NrOfRequiredChunks;

    /* Input data dependent variables */
    GC_CharacterType*                      WeightsTypeString;
    GC_CharacterType*                      WeightsSumTypeString;

    GC_LargestUnsignedIntegerType          WeightsMemorySize;

} GC_CoverageInputDataStruct;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

void InitializeCoverageInputData
    (      GC_CoverageInputDataStruct*  InputData,
     const GC_CoverageParametersStruct* Parameters);

void UpdateCoverageInputData
    (      GC_CoverageInputDataStruct*  InputData,
           GC_CoverageParametersStruct* Parameters);

void ReleaseCoverageSolverInputData
    (      GC_CoverageInputDataStruct*  InputData,
     const GC_InputVectorIndexType      NrOfInputVectors);

/****************************************************************************/

#endif /* GC_COVERAGE_SOLVER_INPUT_DATA_H */