#ifndef GC_COVERAGE_INPUT_DATA_CREATOR_H
#define GC_COVERAGE_INPUT_DATA_CREATOR_H

/****************************************************************************/

#include "CompressedInputVectorsTree.h"
#include "CoverageSolverInputData.h"

/****************************************************************************/
/* Enumerations **************************************************************
/****************************************************************************/

typedef enum {

    GC_VECTOR_LENGTH,
    GC_VECTOR

} GC_VectorCreationEnum;

/****************************************************************************/

typedef enum {

    GC_VECTOR_CREATION_FAILURE,
    GC_VECTOR_CREATION_SUCCESS,
    GC_VECTOR_CREATION_NO_SOLUTION,

} GC_VectorCreationInfoEnum;

/****************************************************************************/
/* Structs ******************************************************************/
/****************************************************************************/

typedef struct {

    GC_LargestUnsignedIntegerType NrOfUVMapValues;

    GC_BitIteratorType            NrOfBytesRequiredPerUVMapValue;
    GC_BitIteratorType            NrOfUVMapValuesPerUVMapChunk;

    GC_LargestUnsignedIntegerType UVMapValueBitMask;
    GC_BitIteratorType            RightShiftSize[sizeof(GC_UVMapChunkType)];

    GC_InputVectorIndexType       TotalNrOfVectors;                           /* Nr of input vectors + 1 (1 for target vector). */

    GC_HostWorkItemType*          CurrentEntriesOfVectors;

} GC_CreatorAuxiliaryVariablesStruct;

/****************************************************************************/

typedef struct {

    const GC_BooleanType*              ComputationStopRequested;

    GC_CoverageParametersStruct*       Parameters;

    GC_CoverageInputDataStruct*        InputData;

    GC_StringSaverStruct*              OutputLogger;

    GC_UVMapsStruct                    TargetUVMaps;
    
    FILE**                             UVMapFiles;

    GC_CreatorAuxiliaryVariablesStruct Aux;

    GC_LargestUnsignedIntegerType      RoundedLengthOfReadVector;
    GC_BitIteratorType                 NrOfBitsInWeightsType;

    /* Required only if input vectors are compressed. */
    GC_LargestUnsignedIntegerType*     UncompressedWeights;
    GC_CompressedInputVectorsTreeNode* CompressedInputVectorsTree;

} GC_CoverageInputDataCreatorStruct;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

/* Code entry function ******************************************************/

GC_BooleanType PrepareCoverageInputData
    (const GC_CoverageParametersStruct*       Parameters,
     const GC_StringSaverStruct*              OutputLogger,
           GC_LargestUnsignedIntegerType*     NrOfUVMapValues,
           GC_CoverageInputDataStruct**       InputData,
     const GC_BooleanType*                    ComputationStopRequested);

/****************************************************************************/

void InitializeCoverageInputDataCreator
    (      GC_CoverageInputDataCreatorStruct* Self,
     const GC_CoverageParametersStruct*       Parameters,
     const GC_StringSaverStruct*              OutputLogger,
     const GC_BooleanType*                    ComputationStopRequested);

void CleanUpCoverageInputDataCreator
    (      GC_CoverageInputDataCreatorStruct* Self);

void ReadTargetUVMapsFromImages
    (      GC_CoverageInputDataCreatorStruct* Self);

void UVMapReadImagesHelperFunction
    (      GC_CoverageInputDataCreatorStruct* Self,
     const GC_InputVectorIndexType            VectorID,
     const GC_BooleanType                     ComputeLength);

GC_VectorCreationInfoEnum ComputeVectorLengthAndPotentiallyCreateTargetUVMap
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_LargestUnsignedIntegerType*     NrOfUVMapValues,
     const GC_BooleanType                     TargetUVMapExists);

void ComputeCompressedVectorLength
    (      GC_CoverageInputDataCreatorStruct* Self);

void FillCompressedVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void AppendZeros
    (      GC_HostWorkItemType*               WorkItem,
           GC_BitIteratorType                 NrOfBits);

void StoreCoverageInputData
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_CoverageInputDataStruct**       InputData);

void SetTargetVectorWorkItem
    (      GC_CoverageInputDataCreatorStruct* Self,
     const GC_InputVectorIndexType            i,
     const GC_LargestUnsignedIntegerType      Index);

void InsertEntriesToInputVector
    (      GC_HostWorkItemType**              Vectors,
     const GC_InputVectorIndexType            TotalNrOfVectors,
     const GC_LargestUnsignedIntegerType      VectorWorkItemID,
     const GC_BitIteratorType                 CurrentOffset,
     const GC_UVMapChunkType*                 CurrentValues,
     const GC_BooleanType                     CompressedVectorIsConstructed);

GC_LargestUnsignedIntegerType FillVectorsAndWeights
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_CompressedInputVectorsTreeNode* Node,
           GC_UVMapChunkType*                 Values,
           GC_LargestUnsignedIntegerType      Counter,
     const GC_LargestUnsignedIntegerType      Level);

void ComputeMaximalPossibleCoverage
    (      GC_CoverageInputDataCreatorStruct* Self);

GC_LargestUnsignedIntegerType Popcount
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_HostWorkItemType                Element,
           GC_LargestUnsignedIntegerType      IndexOfWeight);

void FillVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void CreateVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void SetRemainingVariablesOfCoverageInputData
    (      GC_CoverageInputDataCreatorStruct* Self);

void AllocateAndInitializeVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void OptimizeCompressedVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void FurtherCompressVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void PrepareVectorsForDevice
    (      GC_CoverageInputDataCreatorStruct* Self);

void PrepareVectorForDevice
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_HostWorkItemType*               Vector,
     const GC_LargestUnsignedIntegerType      BitMask);

void SetNrOfBitsInWeightsType
    (      GC_CoverageInputDataCreatorStruct* Self);

void CopyWeights
    (      GC_CoverageInputDataCreatorStruct* Self);

void PrintVectorData
    (      GC_CoverageInputDataCreatorStruct* Self,
           GC_LargestUnsignedIntegerType*     VectorToPrint);

GC_LargestUnsignedIntegerType* GetWeightsPermutation
    (      GC_CoverageInputDataCreatorStruct* Self);

void SortWeightsAndInputVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void RoundWeightsToFrequentValues
    (      GC_CoverageInputDataCreatorStruct* Self);

GC_LargestUnsignedIntegerType RoundValueToClosestPowerOfTwo
    (const GC_LargestUnsignedIntegerType      Value);

void ComputeNrOfDifferentWeights
    (      GC_CoverageInputDataCreatorStruct* Self);

void UpdateNrOfCoverableTargetLocations
    (      GC_CoverageInputDataCreatorStruct* Self);

void ComputeIntervals
    (      GC_CoverageInputDataCreatorStruct* Self);

void OptimizeOrderOfVectors
    (      GC_CoverageInputDataCreatorStruct* Self);

void CopyBit
    (      GC_HostWorkItemType*               VectorToCopyFrom,
           GC_HostWorkItemType*               VectorToCopyTo,
           GC_LargestUnsignedIntegerType      IndexOfBitInVectorToCopyFrom,
           GC_LargestUnsignedIntegerType      IndexOfBitInVectorToCopyTo);

void CopyDeviceWorkItem
    (const GC_BandWidthType                   NrOfBitsInDeviceVectorWorkItem,
           GC_HostWorkItemType*               VectorToCopyFrom,
           GC_HostWorkItemType*               VectorToCopyTo,
           GC_LargestUnsignedIntegerType      IndexOfBitInVectorToCopyFrom,
           GC_LargestUnsignedIntegerType      IndexOfBitInVectorToCopyTo);

GC_LargestUnsignedIntegerType* SortMatrixColumnsAndGetSortPermutation
    (      GC_LargestUnsignedIntegerType***   MatrixToSort,
     const GC_LargestUnsignedIntegerType      Width,
     const GC_LargestUnsignedIntegerType      Height,
     const GC_SortingOrderEnum                SortingOrder,
     const GC_BooleanType                     GetSortPermutation,
     const GC_BooleanType                     MatrixToSortAlreadyTransposed);

void BottomUpMerge
    (      GC_LargestUnsignedIntegerType**    TransposedMatrix,
           GC_LargestUnsignedIntegerType**    MergedTransposedMatrix,
     const GC_LargestUnsignedIntegerType      NrOfEntriesToCompare,
     const GC_LargestUnsignedIntegerType      Left,
     const GC_LargestUnsignedIntegerType      Middle,
     const GC_LargestUnsignedIntegerType      Right,
     const GC_SortingOrderEnum                SortingOrder);

GC_BooleanType Compare
    (      GC_LargestUnsignedIntegerType*     a,
           GC_LargestUnsignedIntegerType*     b,
     const GC_LargestUnsignedIntegerType      NrOfEntriesToCompare,
     const GC_SortingOrderEnum                SortingOrder);

/****************************************************************************/

#endif /* GC_COVERAGE_INPUT_DATA_CREATOR_H */