#ifndef GC_CYCLIC_LIST_H
#define GC_CYCLIC_LIST_H

/****************************************************************************/

#include "StringSaver.h"

/****************************************************************************/
/* GC_CyclicListEntryStruct structure ***************************************/
/****************************************************************************/

struct GC_CyclicListEntryStruct {

           GC_InputVectorIndexType   Index;
    struct GC_CyclicListEntryStruct* Next;
};

typedef struct GC_CyclicListEntryStruct GC_CyclicListEntryStruct;

/****************************************************************************/
/* GC_CyclicListStruct structure ********************************************/
/****************************************************************************/

typedef struct {

    GC_InputVectorIndexType       Length;

    GC_LargestUnsignedIntegerType Value;

    GC_CyclicListEntryStruct*     Tail;                                       /* Always points on tail of cyclic list */

    GC_CyclicListEntryStruct*     CachedEntry;                                /* Cached entry (for efficient updating of list) */

} GC_CyclicListStruct;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

void InitializeList
    (GC_CyclicListStruct*               List);

GC_CyclicListEntryStruct* CreateNewEntry
    (const GC_InputVectorIndexType      Index);

GC_CyclicListEntryStruct* AddIndex
    (      GC_CyclicListStruct*         List,
     const GC_InputVectorIndexType      Index);

GC_BooleanType AddIndexAtPosition
    (      GC_CyclicListStruct*         List,
     const GC_InputVectorIndexType      Index,
     const GC_InputVectorIndexType      Position,
     const GC_BooleanType               PreviousIsInCache);

GC_CyclicListEntryStruct* GetEntryAtPosition
    (      GC_CyclicListStruct*         List,
     const GC_InputVectorIndexType      Position);

GC_CyclicListEntryStruct* GetEntry
    (      GC_CyclicListEntryStruct*    CurrentEntry,
     const GC_InputVectorIndexType      NrOfEntriesLeft);

void PrintList
    (      GC_StringSaverStruct*        OutputLogger,
     const GC_CyclicListStruct*         List,
     const GC_StringSaverPrintFlagEnum  PrintFlags,
     const GC_SensorStruct**            Sensors);

void PrintListEntry
    (      GC_StringSaverStruct*        OutputLogger,
     const GC_CyclicListEntryStruct*    Entry,
     const GC_InputVectorIndexType      NrOfEntriesToPrint,
     const GC_StringSaverPrintFlagEnum  PrintFlags,
     const GC_SensorStruct**            Sensors);

GC_InputVectorIndexType* CopyListToArray
    (const GC_CyclicListStruct*         List);

void EntryToArray
    (      GC_CyclicListEntryStruct*    Entry,
           GC_InputVectorIndexType*     Arr,
     const GC_InputVectorIndexType      NrOfRemainingEntries);

void DeleteList
    (      GC_CyclicListStruct*         List);

void DeleteListEntry
    (      GC_CyclicListEntryStruct*    Entry,
     const GC_InputVectorIndexType      NrOfRemainingEntries);

/****************************************************************************/

#endif /* GC_CYCLIC_LIST_H */