#ifndef GC_HELPER_FUNCTIONS_H
#define GC_HELPER_FUNCTIONS_H

/****************************************************************************/

#include "Image.h"

/****************************************************************************/
/* Definitions **************************************************************/
/****************************************************************************/

#define GC_DIRECTORY_SEPARATOR    "\\"

/* TODO: Delete if not required anymore (work-around for Florin). */
#define GC_MAX_OBJECT_NAME_LENGTH 8

/****************************************************************************/
/* Type definitions *********************************************************/
/****************************************************************************/

typedef cl_uchar GC_SingleColorValue;

/****************************************************************************/
/* Enumerations *************************************************************/
/****************************************************************************/

typedef enum {

    GC_RGBA_UV_MAP,
    GC_IS_TEMPORARY_UV_MAP

} GC_UVMapTypeEnum;

/****************************************************************************/

typedef enum {

    GC_UV_MAP_VALUE_TARGET_VECTOR_INCONSISTENCY,
    GC_UV_MAP_VALUE_INPUT_VECTORS_INCONSISTENCY,
    GC_UV_MAP_VALUE_DONT_CARE,
    GC_UV_MAP_VALUE_INVALID,
    GC_UV_MAP_VALUE_CANNOT_BE_REACHED,
    GC_UV_MAP_VALUE_NOT_REQUIRED,
    GC_UV_MAP_VALUE_REQUIRED

} GC_UVMapValueNecessityEnum;

/****************************************************************************/

typedef enum {

    GC_VECTOR_ENTRY_NOT_SET,
    GC_VECTOR_ENTRY_SET

} GC_InputVectorValueEnum;

/****************************************************************************/
/* Structs ******************************************************************/
/****************************************************************************/

typedef struct {

    GC_UVMapChunkType CurrentUVMapValue;
    GC_UVMapChunkType CurrentTargetUVMapValue;
    GC_UVMapChunkType MaximalEntryOfUnitedVector;
    GC_UVMapChunkType MaximalTargetVectorEntry;

} GC_UVMapValueCollectionStruct;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

GC_LargestUnsignedIntegerType SmallestMultipleOfANotSmallerThanB
    (const GC_LargestUnsignedIntegerType a,
     const GC_LargestUnsignedIntegerType b);

GC_LargestUnsignedIntegerType GreatestMultipleOfANotGreaterThanB
    (const GC_LargestUnsignedIntegerType a,
     const GC_LargestUnsignedIntegerType b);

GC_BooleanType ComputeFileSize
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              FileName,
           GC_LargestUnsignedIntegerType* FileSize);

GC_BooleanType DeleteFile
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              Path);

GC_BooleanType CreateDirectory
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              Path);

GC_BooleanType DeleteDirectory
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              Path);

GC_BooleanType OpenUVMapImage
    (      GC_PngImageStruct*             PngImage,
     const GC_GeneralParametersStruct*    GeneralParameters,
     const GC_InputVectorIndexType        VectorID,
     const GC_NrOfObjectsType             ObjectID,
     const GC_StringSaverStruct*          OutputLogger);

void CreateFileName
    (      GC_CharacterType*              FileName,
     const GC_CharacterType*              Path,
     const GC_GeneralParametersStruct*    GeneralParameters,
     const GC_InputVectorIndexType        SensorID,
     const GC_NrOfViewsType               ViewID,
     const GC_NrOfObjectsType             ObjectID,
     const GC_Space                       CurrentSpace,
     const GC_UVMapTypeEnum               UVMapType);

GC_BooleanType DoesDirectoryExist
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              Path,
     const GC_BooleanType                 DirectoryShouldExist);

GC_BooleanType DoesFileExist
    (const GC_StringSaverStruct*          OutputLogger,
     const GC_CharacterType*              Path,
     const GC_BooleanType                 FileShouldExist,
     const GC_BooleanType                 ShowWarnings);

GC_BitIteratorType GetNrOfBytesRequiredPerUVMapValue
    (const GC_PrecisionType               Precision);

GC_BitIteratorType GetNrOfUVMapValuesPerUVMapChunk
    (const GC_PrecisionType               Precision);

GC_BitIteratorType GetUVMapValueShiftSize
    (const GC_PrecisionType               Precision);

void SetRightShiftSizeArray
    (      GC_BitIteratorType             RightShiftSize[sizeof(GC_UVMapChunkType)],
     const GC_PrecisionType               Precision);

GC_LargestUnsignedIntegerType GetUVMapValueBitMask
    (const GC_PrecisionType               Precision);

GC_LargestUnsignedIntegerType ConvertUVMapValueToRGBAValue
    (const GC_PrecisionType               Precision,
     const GC_VisualizationSettingsStruct VisualizationSettings,
     const GC_UVMapValueCollectionStruct  Values);

GC_LargestUnsignedIntegerType GetColorFromColorMap
    (const GC_InputVectorEntryType        Value,
     const GC_VisualizationTypeEnum       VisualizationType);

void ComputeColor
    (const GC_InputVectorEntryType        x,
     const cl_double                      UpperLimit2Factor,
     const cl_double                      UpperLimit1Factor,
     const GC_InputVectorEntryType        y,
     const cl_double                      LowerLimit1Factor,
     const cl_double                      LowerLimit2Factor,
     const GC_VisualizationTypeEnum       VisualizationType,
           GC_SingleColorValue*           r,
           GC_SingleColorValue*           g,
           GC_SingleColorValue*           b);

GC_InputVectorEntryType ConvertUVMapValueToVectorEntryValue
    (const GC_PrecisionType               Precision,
     const GC_UVMapChunkType              UVMapValue,
           GC_BooleanType*                IsDontCare,
           GC_BooleanType*                InvalidUVMapValue);

GC_UVMapChunkType ConvertVectorEntryValueToUVMapValue
    (const GC_PrecisionType               Precision,
     const GC_InputVectorEntryType        VectorEntry);

GC_UVMapValueNecessityEnum FillEntriesArray
    (      GC_UVMapChunkType*             CurrentEntryValueOfVector,
     const GC_UVMapsStruct*               UVMaps,
     const GC_LargestUnsignedIntegerType  UVMapChunkID,
     const GC_LargestUnsignedIntegerType  TargetVectorWorkItemID,
     const cl_uchar                       RightShiftSize,
     const GC_LargestUnsignedIntegerType  UVMapValueBitMask,
     const GC_PrecisionType               Precision,
     const GC_InputVectorIndexType        NrOfInputVectors,
     const GC_BooleanType                 TargetUVMapExists,
     const GC_BooleanType                 InputDataCreation,
     const GC_BooleanType                 StrictConsistency,
     const cl_double                      LowerBoundFactor,
     const cl_double                      UpperBoundFactor);

GC_BooleanType WriteStringToFile
    (const GC_CharacterType*              Path,
     const GC_CharacterType*              StringToWrite);

GC_BooleanType OpenBinaryFileWriteAndClose
    (const GC_CharacterType*              Path,
     const void*                          Data,
     const size_t                         SizeOfElement,
     const size_t                         NrOfElements,
     const GC_BooleanType                 Append);

GC_BooleanType OpenBinaryFileReadAndClose
    (const GC_CharacterType*              Path,
           void*                          Data,
     const size_t                         SizeOfElement,
     const size_t                         NrOfElements);

void PrintCoverageInformation
    (const GC_StringSaverStruct*          OutputLogger,
     const cl_double                      ValueForSatisfiableCoverageConstraints,
     const cl_double                      ValueForAllCoverageConstraints);

GC_BooleanType CheckDirectories
    (const GC_GeneralParametersStruct*    GeneralParameters,
     const GC_StringSaverStruct*          OutputLogger);

GC_BooleanType CreateDirectoryStructure
    (const GC_GeneralParametersStruct*    GeneralParameters,
     const GC_StringSaverStruct*          OutputLogger);

GC_BooleanType DeleteStoredUVMapsFiles
    (const GC_GeneralParametersStruct*    GeneralParameters,
     const GC_StringSaverStruct*          OutputLogger);

/****************************************************************************/

#endif /* GC_HELPER_FUNCTIONS_H */