#ifndef GC_IMAGES_H
#define GC_IMAGES_H

/****************************************************************************/

#include <libpng/png.h>

#include "Definitions.h"
#include "StringSaver.h"

/****************************************************************************/
/* Definitions **************************************************************/
/****************************************************************************/

#define GC_IMAGE_BYTES_TO_CHECK 8

/****************************************************************************/
/* Type definitions *********************************************************/
/****************************************************************************/

typedef cl_int GC_ColorTypeType; /* sic! */
typedef cl_int GC_BitDepthType;
typedef cl_int GC_NrOfChannelsType;

/****************************************************************************/
/* Enumerations *************************************************************/
/****************************************************************************/

typedef enum {

    GC_IMAGE_OPEN_READ_FILE,
    GC_IMAGE_OPEN_WRITE_FILE,
    GC_IMAGE_CREATE_READ_WRITE_STRUCT,
    GC_IMAGE_CREATE_READ_INFO_STRUCT,
    GC_IMAGE_CREATE_WRITE_INFO_STRUCT

} GC_ImageInitializationType;

/****************************************************************************/
/* Structs ******************************************************************/
/****************************************************************************/

typedef struct {

    GC_BooleanType                IsInitialized;

    GC_BooleanType                ReadAccess;

    GC_CharacterType              FileName[GC_MAX_FILE_NAME_LENGTH];
    FILE*                         File;
    png_structp                   PngPointer;
    png_infop                     InfoPointer;

    GC_NrOfChannelsType           NrOfChannels;
    GC_ImageSizeType              Width;
    GC_ImageSizeType              Height;
    GC_BitDepthType               BitDepth;
    GC_ColorTypeType              ColorType;
    cl_int                        InterlaceMethod;
    cl_int                        CompressionMethod;
    cl_int                        FilterMethod;

} GC_PngImageStruct;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

GC_BooleanType InitializeImage
    (      GC_PngImageStruct*         Self,
     const GC_ImageInitializationType InitializationCase,
     const GC_StringSaverStruct*      OutputLogger);

GC_BooleanType ErrorOccurredInitializingPNGFile
    (      GC_PngImageStruct*         Self,
     const GC_ImageInitializationType CaseToCheck,
     const GC_StringSaverStruct*      OutputLogger);

void CloseImage
    (      GC_PngImageStruct*         Self);

void ReadImage
    (      GC_PngImageStruct*         Self,
           png_byte*                  ImageData);

void WriteImage
    (      GC_PngImageStruct*         Self,
     const png_byte*                  ImageData,
     const GC_BooleanType             ReverseYOrder);

#endif /* GC_IMAGES_H */