#ifndef GC_STRING_SAVER_H
#define GC_STRING_SAVER_H

/****************************************************************************/

#include "StopWatch.h"

/****************************************************************************/

#define GC_SAVED_STRING_SIZE              (8192u)

#define GC_MAX_PREFIX_STRING_LENGTH       (255u)
#define GC_MAX_INDENTATION_STRING_LENGTH  (21u)
#define GC_INDENTATION_STEP_SIZE          (4u)
#define GC_INDENTATION_CHARACTER          ' '

#define GC_NO_PREFIX                      ""
#define GC_TIME_STRING_SUFFIX             " | "
#define GC_NO_TIME_STRING_SUFFIX          "                      | "
#define GC_NO_INDENTATION_STRING          ""

#define GC_STARTED_STRING                 "Started:   "
#define GC_FINISHED_STRING                "Finished: "

/****************************************************************************/

typedef cl_uchar GC_IndentationPrefixLengthType;
typedef cl_uchar GC_IndentationStringLengthType;

typedef enum {

    GC_SHOW_OUTPUT_TIME = 0x1,
    GC_SHOW_PREFIX      = 0x2,
    GC_TALKATIVE        = 0x4,

} GC_StringSaverPrintFlagEnum;

/****************************************************************************/
/* GC_StringSaverStruct structure *******************************************/
/****************************************************************************/

typedef struct {

    GC_BooleanType                 LogToExtern;
    GC_BooleanType                 IsTalkative;

    GC_LargestUnsignedIntegerType  StringLength;
    GC_LargestUnsignedIntegerType  CurrentMaxStringLength;
    GC_IndentationPrefixLengthType PrefixLength;

    GC_CharacterType               Prefix[GC_MAX_PREFIX_STRING_LENGTH];       /* Length has to fit into GC_IndentationPrefixLengthType. */
    GC_CharacterType               Indentation[GC_MAX_INDENTATION_STRING_LENGTH];
    GC_CharacterType*              FixedIndentation;
    GC_CharacterType*              String;

    const GC_StopWatchStruct       StopWatch;

    const void                     (*Printf)(GC_CharacterType*,
                                             GC_MessageKind);

} GC_StringSaverStruct;

/****************************************************************************/

void AppendToString
    (      GC_StringSaverStruct*            Self,
     const GC_CharacterType*                StringToAppend,
     const GC_LargestUnsignedIntegerType    LengthOfStringToAppend);

void DecreaseIndentation
    (      GC_StringSaverStruct*            Self);

void ReleaseStringSaver
    (      GC_StringSaverStruct*            Self);

void IncreaseIndentation
    (      GC_StringSaverStruct*            Self);

void InitializeStringSaver
    (      GC_StringSaverStruct*            Self,
     const GC_BooleanType                   LogToExtern,
     const GC_BooleanType                   IsTalkative,
     const GC_CharacterType*                FixedIndentation,
     const GC_CharacterType*                Prefix,
     const void                             (*Printf)(GC_CharacterType*,
                                                      GC_MessageKind));
void Print
    (const GC_StringSaverStruct*            Self,
     const GC_StringSaverPrintFlagEnum      Flags,
     const GC_MessageKind                   MessageKind,
     const GC_CharacterType*                StringToSave,
                                            ...);

void Save
    (      GC_StringSaverStruct*            Self,
     const GC_CharacterType*                StringToSave,
                                            ...);

void UpdateStringSaver
    (const GC_StringSaverStruct*            Self,
     const GC_StringSaverPrintFlagEnum      Flags,
     const GC_MessageKind                   MessageKind,
     const GC_CharacterType*                StringToSave,
           va_list                          Arguments);

GC_BooleanType WriteToFile
    (const GC_StringSaverStruct*            Self,
     const GC_CharacterType*                Path);

/****************************************************************************/

#endif /* GC_STRING_SAVER_H */