#ifndef GC_DEFINITIONS_H
#define GC_DEFINITIONS_H

/****************************************************************************/

#include "GC.h"

/****************************************************************************/
/** Definitions *************************************************************/
/****************************************************************************/

/* Platform dependent definitions *******************************************/

#ifdef WIN32
    #define GC_NR_OF_WORK_ITEMS_TYPE_STRING "uint"
    typedef cl_uint                         GC_NrOfWorkItemsType;
#else /* 64 bit system */
    #define GC_NR_OF_WORK_ITEMS_TYPE_STRING "ulong"
    typedef cl_ulong                         GC_NrOfWorkItemsType;
#endif

/* Platform independent definitions *****************************************/

#define GC_NOT_REQUIRED                   0

#define GC_NR_OF_BITS_IN_BYTE             (8u)
#define GC_MAX_VALUE_OF_UBYTE             (0xFF)

#define GC_NR_OF_BITS_IN_CL_UCHAR         GC_NR_OF_BITS_IN_BYTE
#define GC_NR_OF_BITS_IN_CL_USHORT        (16u)
#define GC_NR_OF_BITS_IN_CL_UINT          (32u)
#define GC_NR_OF_BITS_IN_CL_ULONG         (64u)

#define GC_MAX_VALUE_OF_CL_UCHAR          GC_MAX_VALUE_OF_UBYTE
#define GC_MAX_VALUE_OF_CL_USHORT         (0xFFFF)
#define GC_MAX_VALUE_OF_CL_INT            (0x7FFFFFFF)
#define GC_MAX_VALUE_OF_CL_UINT           (0xFFFFFFFF)
#define GC_MAX_VALUE_OF_CL_ULONG          (0xFFFFFFFFFFFFFFFF)

#define GC_BOOLEAN_TYPE_STRING            "uchar"

#define GC_INPUT_VECTOR_INDEX_TYPE_STRING "int"

#define GC_MAX_FILE_NAME_LENGTH           (1000u)

#define GC_MAX_VALUE_BYTE_PRECISION       GC_MAX_VALUE_OF_CL_UCHAR - 1
#define GC_MAX_VALUE_SHORT_PRECISION      GC_MAX_VALUE_OF_CL_USHORT - 1

#define GC_FILE_EXTENSION                 "png"

#define GC_COVERAGE_PREFIX                "GC | "
#define GC_RGBA_MAP_CREATION_PREFIX       "GC | map creation | "

#define GC_MAX_INPUT_VECTOR_ENTRY         (0xFFFFFFFF)

#define GC_PERCENTAGE_FACTOR              100

#define GC_NO_UPPER_BOUND_VALUE           (1e7)

/* Preprocessor function definitions ****************************************/

#define GC_GET_PRECISION(x)       (x -> GeneralParameters -> Precision)
#define GC_IS_BIT_PRECISION(x)    (GC_BIT_PRECISION   == GC_GET_PRECISION(x))
#define GC_IS_BYTE_PRECISION(x)   (GC_BYTE_PRECISION  == GC_GET_PRECISION(x))
#define GC_IS_SHORT_PRECISION(x)  (GC_SHORT_PRECISION == GC_GET_PRECISION(x))
#define GC_IS_INT_PRECISION(x)    (GC_INT_PRECISION   == GC_GET_PRECISION(x))

#define GC_BINARY_SENSORS(x)      (x -> GeneralParameters -> BinarySensors)

#define GC_GET_MAIN_DIR(x)        (x -> GeneralParameters -> MainDirectoryPath)
#define GC_GET_WORK_DIR(x)        (x -> GeneralParameters -> WorkDirectoryPath)
#define GC_GET_INPUT_DIR(x)       (x -> GeneralParameters -> InputDirectoryPath)

#define GC_GET_NR_OF_TARGETS(x)       (x -> GeneralParameters -> NrOfTargets)
#define GC_GET_NR_OF_OCCLUDERS(x)     (x -> GeneralParameters -> NrOfOccluders)
#define GC_GET_NR_OF_SENSORS(x)       (x -> GeneralParameters -> NrOf.Sensors)
#define GC_GET_NR_OF_INPUT_VECTORS(x) (x -> GeneralParameters -> NrOf.InputVectors)
#define GC_READ_TARGET_VECTOR(x)      (x -> GeneralParameters -> ReadTargetVector)

#define GC_STRICT_CONSISTENCY(x)      (x -> GeneralParameters -> StrictConsistency)

#define GC_IS_LAST_SENSOR(s, x)   (s == x -> GeneralParameters -> NrOfSensors - 1)
#define GC_IS_LAST_VIEW(v, s, x)  (v == x -> Sensors[s] -> NrOfViews - 1)
#define GC_IS_LAST_OBJECT(o, x)   (o == x -> GeneralParameters -> NrOfObjects - 1)

#define GC_DEVICE_SCALAR_WORK_ITEM_BIT_SIZE(x)    (x -> Optimization.NrOfBitsInDeviceScalarWorkItem)
#define GC_DEVICE_VECTOR_WORK_ITEM_CARDINALITY(x) (x -> Optimization.DeviceVectorWorkItemCardinality)

#define GC_FREE_AND_SET_TO_NULL(x)    if(x != NULL) { free(x); x = NULL; }

/* UV map values ************************************************************/

#define GC_UV_MAP_DONT_CARE_VALUE                    0

#define GC_UV_MAP_NOT_SET_VALUE_BIT_PRECISION        (30u)
#define GC_UV_MAP_SET_VALUE_BIT_PRECISION            (255u)

#define GC_UV_MAP_NOT_SET_VALUE_NOT_BIT_PRECISION    (1u)

#define GC_UV_MAP_SET_VALUE_BYTE_PRECISION           (GC_MAX_VALUE_OF_UBYTE  - GC_UV_MAP_NOT_SET_VALUE_BYTE_SHORT_PRECISION)
#define GC_UV_MAP_SET_VALUE_SHORT_PRECISION          (GC_MAX_VALUE_OF_USHORT - GC_UV_MAP_NOT_SET_VALUE_BYTE_SHORT_PRECISION)

/* XYZ **********************************************************************/

#define GC_XYZ_FILENAME_PART_0            "XYZ_SENSOR_"
#define GC_XYZ_FILENAME_PART_1            "_VIEW_"

/* UV ***********************************************************************/

#define GC_FILENAME_PART_0_UV             "SENSOR_"
#define GC_FILENAME_PART_1_UV             "_OBJECT_"

#define GC_FILENAME_TARGET                "TARGET"
#define GC_FILENAME_SOLUTION              "SOLUTION_"

/****************************************************************************/
/* Type definitions *********************************************************/
/****************************************************************************/

typedef cl_uint                       GC_InputVectorEntryType;
typedef cl_char                       GC_BitIteratorSignedType;
typedef GC_LargestUnsignedIntegerType GC_HostWorkItemType;

/****************************************************************************/
/* Functions ****************************************************************/
/****************************************************************************/

void DestroyGeneralParameters
    (      GC_GeneralParametersStruct**     GeneralParameters);

void GC_ResetCoverageSolutionStruct
    (      GC_CoverageSolutionStruct*       CoverageSolution);

/****************************************************************************/

#endif /* GC_DEFINITIONS_H */