#ifndef GC_SHADERS_H
#define GC_SHADERS_H

/****************************************************************************/
/* XYZ vertex shader ********************************************************/
/****************************************************************************/

static const GC_CharacterType* XYZVertexShader =

"#version 410                                                                 \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
in dvec3 vPositionXYZ;                                                        \n\
in uint vId;                                                                  \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
uniform dmat4 View;                                                           \n\
uniform dmat4 Model;                                                          \n\
uniform dmat4 Projection;                                                     \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
flat out uint fId; /* Using flat since no interpolation should be performed.*/\n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
void main()                                                                   \n\
{                                                                             \n\
    /* Make sure that minimal number of operations are used (matrix vector  */\n\
    /* multiplications instead of matrix matrix multiplications).           */\n\
                                                                              \n\
    gl_Position = vec4(Projection * (View * dvec4(vPositionXYZ, 1)));         \n\
    fId         = vId;                                                        \n\
}";

/****************************************************************************/
/* XYZ fragment shader ******************************************************/
/****************************************************************************/

static const GC_CharacterType* XYZFragmentShader =
"#version 410                                                                 \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
flat in uint fId; /* Using flat since no interpolation should be performed. */\n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
layout(location = 0) out uint resultId;                                       \n\
                                                                              \n\
layout(pixel_center_integer) in vec4 gl_FragCoord;                            \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
void main()                                                                   \n\
{                                                                             \n\
    resultId = fId;                                                           \n\
    //resultId = floatBitsToUint(gl_FragCoord.z);                               \n\
}";

/****************************************************************************/
/* UV vertex shader *********************************************************/
/****************************************************************************/

static const GC_CharacterType* UVVertexShader =
"#version 410                                                                 \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
in dvec2 vPositionUV;                                                         \n\
in dvec3 vPositionXYZ;                                                        \n\
in dvec3 vNormal;                                                             \n\
in uint  vId;                                                                 \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
uniform dmat4 View;                                                           \n\
uniform dmat4 Model;                                                          \n\
uniform dmat4 Projection;                                                     \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
     out vec3 fPositionXYZ;          /* XYZ (world) coordinates after       */\n\
                                     /* applying view transformation.       */\n\
                                                                              \n\
     out vec3 fProjectedPositionXYZ; /* Screen coordinates */                 \n\
                                                                              \n\
     out vec3 fNormal;               /* Normal coordinates */                 \n\
                                                                              \n\
flat out uint fId;                   /* Using flat since no interpolation   */\n\
                                     /* should be performed                 */\n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
void main()                                                                   \n\
{                                                                             \n\
    dvec4 TempPositionXYZ;                                                    \n\
    dvec4 TempNormal;                                                         \n\
                                                                              \n\
    /* Compute the correct UV coordinates for mapping to viewport. Note     */\n\
    /* that the UV coordinates are in the interval [0,1]. However, in order */\n\
    /* to correctly map them onto the viewport, they have to be in the      */\n\
    /* interval [-1,1].                                                     */\n\
                                                                              \n\
    gl_Position = vec4(vPositionUV.x * 2 - 1, vPositionUV.y * 2 - 1, 0, 1);   \n\
                                                                              \n\
    /* XYZ coordinates */                                                     \n\
    TempPositionXYZ       = View * dvec4(vPositionXYZ, 1);                    \n\
    fPositionXYZ          = vec3(TempPositionXYZ / TempPositionXYZ.w);        \n\
    TempPositionXYZ       = Projection * TempPositionXYZ;                     \n\
    TempPositionXYZ       = TempPositionXYZ / TempPositionXYZ.w;              \n\
    fProjectedPositionXYZ = vec3(TempPositionXYZ.xyz);                        \n\
                                                                              \n\
    /* Normals */                                                             \n\
    fNormal =                                                                 \n\
        normalize(  vec3(View * dvec4(vPositionXYZ + vNormal, 1)              \n\
                  - View * dvec4(vPositionXYZ, 1)));                          \n\
                                                                              \n\
    /* IDs */                                                                 \n\
    fId = vId;                                                                \n\
}";

/****************************************************************************/
/* UV fragment shader *******************************************************/
/****************************************************************************/

static const GC_CharacterType* UVFragmentShader =
"#version 410                                                                 \n\
#extension GL_EXT_gpu_shader4 : enable                                        \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
#define GC_MAX_VALUE_OF_UBYTE                        (255u)                   \n\
#define GC_MAX_VALUE_OF_USHORT                       (65535u)                 \n\
                                                                              \n\
#define GC_UV_MAP_DONT_CARE_VALUE                    0                        \n\
                                                                              \n\
#define GC_UV_MAP_NOT_SET_PIXEL_BIT_PRECISION        (30u)                    \n\
#define GC_UV_MAP_SET_PIXEL_BIT_PRECISION            (255u)                   \n\
                                                                              \n\
#define GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION (1u)                     \n\
                                                                              \n\
#define GC_UV_MAP_MIN_SET_PIXEL_BYTE_SHORT_PRECISION (GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION + 1)\n\
                                                                              \n\
#define GC_UV_MAP_MAX_SET_PIXEL_BYTE_PRECISION       (GC_MAX_VALUE_OF_UBYTE  - GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION)\n\
#define GC_UV_MAP_MAX_SET_PIXEL_SHORT_PRECISION      (GC_MAX_VALUE_OF_USHORT - GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION)\n\
                                                                              \n\
#define GC_BIT_PRECISION                             0                        \n\
#define GC_BYTE_PRECISION                            1                        \n\
#define GC_SHORT_PRECISION                           2                        \n\
                                                                              \n\
#define GC_DOT_PRODUCT_THRESHOLD                     (0.02)                   \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
     in vec3 fPositionXYZ;                                                    \n\
     in vec3 fProjectedPositionXYZ;                                           \n\
     in vec3 fNormal;                                                         \n\
flat in uint fId;  /* Using flat since no interpolation should be performed.*/\n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
uniform float Factor;                                                         \n\
uniform uint  Precision;                                                      \n\
uniform uint  BinarySensors;                                                  \n\
uniform uint  IsFirstView;                                                    \n\
uniform uint  SideLengthOfXYZMap;                                             \n\
uniform uint  NeighborhoodRadius;                                             \n\
                                                                              \n\
uniform usampler2D XYZMapTexture; /* Attached to GL_COLOR_ATTACHMENT0. */     \n\
uniform usampler2D UVMapTexture;  /* Attached to GL_COLOR_ATTACHMENT2. */     \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
/* Important note: ResultUVMapValue is attached to GL_COLOR_ATTACHMENT1.    */\n\
/*                 Nevertheless, location has to be 0 since the actual      */\n\
/*                 correspondence is set from the host using glDrawBuffer.  */\n\
layout(location = 0) out uint ResultUVMapValue;                               \n\
                                                                              \n\
layout(pixel_center_integer) in vec4 gl_FragCoord;                            \n\
                                                                              \n\
/****************************************************************************/\n\
                                                                              \n\
/* NOTE: Although usually allowed in C, const variables with values unkown  */\n\
/* during compile time are forbidden in some older versions of GLSL and     */\n\
/* hence even variables which should be const in regular C are non-const    */\n\
/* in the following function.                                               */\n\
                                                                              \n\
void main()                                                                   \n\
{                                                                             \n\
    bool IdMatch               = false;                                       \n\
    uint NeighborhoodDiameter  = 2 * NeighborhoodRadius + 1;                  \n\
    uint NumberOfCheckedPixels = NeighborhoodDiameter * NeighborhoodDiameter; \n\
                                                                              \n\
    /************************************************************************/\n\
    /* Check the corresponding pixel in the XYZ map and its                 */\n\
    /* NumberOfCheckedPixels - 1 neighbors.                                 */\n\
    /************************************************************************/\n\
                                                                              \n\
    for(uint i = 0; i < NumberOfCheckedPixels; i++) {                         \n\
                                                                              \n\
        int   LeftShift;                                                      \n\
        int   TopShift;                                                       \n\
        float Margin;                                                         \n\
        vec2  ProjectedXYZPosition;                                           \n\
        uint  IdOfTriangleAtPixel;                                            \n\
                                                                              \n\
        LeftShift = int(  (i % NeighborhoodDiameter)                          \n\
                        - NeighborhoodRadius);                                \n\
                                                                              \n\
        TopShift = int(  (i / NeighborhoodDiameter)                           \n\
                       - NeighborhoodRadius);                                 \n\
                                                                              \n\
        Margin = 1.0 / float(SideLengthOfXYZMap);                             \n\
                                                                              \n\
        /* Compute the correct texture coordinates of the corresponding     */\n\
        /* pixel in the XYZ map according. Note that the pixel is given in  */\n\
        /* viewport coordinates which are in the interval [-1,1]. However,  */\n\
        /* texture coordinates lie in the interval [0,1].                   */\n\
                                                                              \n\
        ProjectedXYZPosition =                                                \n\
            vec2((fProjectedPositionXYZ.x + 1) / 2 + LeftShift * Margin,      \n\
                 (fProjectedPositionXYZ.y + 1) / 2 + TopShift * Margin);      \n\
                                                                              \n\
        IdOfTriangleAtPixel = texture(XYZMapTexture, ProjectedXYZPosition).x; \n\
                                                                              \n\
        IdMatch = IdMatch || bool(fId == IdOfTriangleAtPixel);                \n\
    }                                                                         \n\
                                                                              \n\
    /************************************************************************/\n\
    /* Set the initial pixel value to \"not set\".                          */\n\
    /************************************************************************/\n\
                                                                              \n\
    /* Use multiplication rather than if-clauses for speeding up. */          \n\
    ResultUVMapValue =                                                        \n\
            uint(GC_BIT_PRECISION == Precision)                               \n\
          * GC_UV_MAP_NOT_SET_PIXEL_BIT_PRECISION                             \n\
        +   uint(GC_BIT_PRECISION != Precision)                               \n\
          * GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION;                     \n\
                                                                              \n\
    /************************************************************************/\n\
    /* If the corresponding pixel (or one of its NumberOfCheckedPixels - 1) */\n\
    /* neighbors) in the XYZ map belongs to the triangle the current        */\n\
    /* fragment belongs to, then update the pixel value.                    */\n\
    /************************************************************************/\n\
                                                                              \n\
    if(   IdMatch                                                             \n\
       &&   dot(normalize(fNormal), normalize(fPositionXYZ))                  \n\
          > GC_DOT_PRODUCT_THRESHOLD) {                                       \n\
                                                                              \n\
        if(GC_BIT_PRECISION == Precision) {                                   \n\
                                                                              \n\
            ResultUVMapValue = GC_UV_MAP_SET_PIXEL_BIT_PRECISION;             \n\
        }                                                                     \n\
        else {                                                                \n\
            float Distance = length(fPositionXYZ);                            \n\
                                                                              \n\
            /* Use multiplication rather than if-clauses for speeding up. */  \n\
            uint TempResultUVMapValue =                                       \n\
                    uint(GC_BYTE_PRECISION == Precision)                      \n\
                  * GC_UV_MAP_MAX_SET_PIXEL_BYTE_PRECISION                    \n\
                +   uint(GC_SHORT_PRECISION == Precision)                     \n\
                  * GC_UV_MAP_MAX_SET_PIXEL_SHORT_PRECISION;                  \n\
                                                                              \n\
            if(bool(BinarySensors)) {                                         \n\
                                                                              \n\
                ResultUVMapValue =                                            \n\
                    GC_UV_MAP_MIN_SET_PIXEL_BYTE_SHORT_PRECISION;             \n\
            }                                                                 \n\
            else {                                                            \n\
                ResultUVMapValue =                                            \n\
                      GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION            \n\
                    + uint(round(  TempResultUVMapValue                       \n\
                                 * (1.0f / (Factor * (1.0 + Distance * Distance)))\n\
                                 * dot(normalize(fNormal), normalize(fPositionXYZ))));\n\
            }                                                                 \n\
        }                                                                     \n\
    }                                                                         \n\
                                                                              \n\
    /************************************************************************/\n\
    /* Update pixel value in case that this is not the first time this UV   */\n\
    /* map is processed.                                                    */\n\
    /************************************************************************/\n\
                                                                              \n\
    if(   !bool(IsFirstView)                                                  \n\
       && (      GC_BIT_PRECISION == Precision                                \n\
              && GC_UV_MAP_NOT_SET_PIXEL_BIT_PRECISION == ResultUVMapValue    \n\
           ||    GC_BIT_PRECISION != Precision                                \n\
              &&    GC_UV_MAP_NOT_SET_PIXEL_BYTE_SHORT_PRECISION              \n\
                 == ResultUVMapValue)) {                                      \n\
                                                                              \n\
        ivec2 PositionOfPixelOnScreen;                                        \n\
        uint  PreviousUVMapValue;                                             \n\
                                                                              \n\
        PositionOfPixelOnScreen = ivec2(gl_FragCoord.xy);                     \n\
                                                                              \n\
        PreviousUVMapValue =                                                  \n\
            texelFetch(UVMapTexture, PositionOfPixelOnScreen, 0).x;           \n\
                                                                              \n\
        ResultUVMapValue =                                                    \n\
                uint(GC_BIT_PRECISION == Precision)                           \n\
              * PreviousUVMapValue                                            \n\
            +   uint(GC_BIT_PRECISION != Precision)                           \n\
              * PreviousUVMapValue;                                           \n\
    }                                                                         \n\
}";

/****************************************************************************/

#endif /* GC_SHADERS_H */